/* file : query-gen.js
MIT License

Copyright (c) 2019 Thomas Minier

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

'use strict'

import { translate } from 'sparqlalgebrajs'
import * as t from 'graphql-ast-types';

/**
 * Try to convert a SPARQL query into a GraphQL query (with its json-ld context)
 * @param  {string} query - SPARQL query
 * @return {string[]|null} A tuple [GraphQL query, JSON-LD context] or `null` if the original query cannot be converted
 */
export default function fromSPARQL (query) {
  const algebra = translate(query)
  if (!isConvertible(algebra)) {
    return null
  }
  const aliases = new Map()
  const bgp = algebra.input.patterns.map(p => {
    return {
      subject: termToContext(p.subject),
      predicate: termToContext(p.predicate),
      object: termToContext(p.object)
    }
  })
}

/**
 * Test if a SPARQL query is convertible to GraphQL one
 * @param  {Object}  algebra - Parsed SPARQL query
 * @return {Boolean} True if a SPARQL query is convertible to GraphQL one, False otherwise
 */
function isConvertible (algebra) {
  return algebra.input.type === 'bgp'
}

function termToContext (term, aliases) {
  if (isIRI(term) && !context.has(term)) {
    // try to split on url
    let index = term.lastIndexOf('#')
    if (index < 0) {
      index = term.lastIndexOf('/')
    }
    const alias = term.substring(index + 1)
    context.set(term, alias)
  }
  return term
}

function isIRI (term) {
  return !term.startsWith('"') && !term.startsWith('?') && !term.startsWith('_')
}
